# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-31 01:32
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0006_auto_20160826_2039'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filernamecd',
            name='naml',
            field=calaccess_raw.fields.CharField(db_column='NAML', help_text='Last name, sometimes full name in the case of PACs, firms and employers. Major donors can be split between first and last name fields, but usually are contained in the last name field only. Individual names of lobbyists, politicans and officers tend to use both the first and last name.', max_length=200),
        ),
    ]
